import amqp, { Options } from "amqplib";
import { ITodo } from "../schemas/todoSchema";
import { EnvService } from "../services/EnvService";
import { DateService } from "../services/DateService";

export class RabbitMQ {
  connection: amqp.Connection;
  channel: amqp.Channel;
  queue: string;
  exchange: string;
  envService: EnvService;
  currentDate: DateService;

  constructor() {
    this.currentDate = DateService.getInstance();
    this.envService = EnvService.getInstance();
    this.queue = this.envService.getEnvVariable("RABBITMQ_QUEUE_NAME");
    this.exchange = "delayed_exchange";

    this.connect()
      .then(() => {
        console.log("RabbitMQ connected");
      })
      .catch((error) => {
        console.error("Error connecting to RabbitMQ:", error);
      });
  }

  async connect() {
    try {
      this.connection = await amqp.connect({
        protocol: "amqp",
        hostname: this.envService.getEnvVariable("RABBITMQ_HOST"),
        port: parseInt(this.envService.getEnvVariable('RABBITMQ_PORT')),
        username: this.envService.getEnvVariable('RABBITMQ_USERNAME'),
        password: this.envService.getEnvVariable('RABBITMQ_PASSWORD'),
      });

      this.channel = await this.connection.createChannel();
      await this.channel.assertQueue(this.queue);
      await this.channel.assertExchange(this.exchange, "x-delayed-message", {
        durable: true,
        arguments: {
          "x-delayed-type": "direct",
        },
      });
      await this.channel.bindQueue(this.queue, this.exchange, this.queue);

    } catch (error) {
      throw error;
    }
  }

  async create(payload: ITodo) {
    const delayTimeForQueue = this.calculateDelayTimeForQueue(payload);
    const message = JSON.stringify({ payload });
    const options: Options.Publish = {
      headers: { "x-delay": delayTimeForQueue },
    };
    try {
      this.channel.publish(
        this.exchange,
        this.queue,
        Buffer.from(message),
        options
      );
    } catch (error) {
      throw error;
    }
  }

  calculateDelayTimeForQueue(payload: ITodo) {
    const delayTime = payload.due_date.getTime() - this.currentDate.getCurrentDate().getTime();
    return delayTime;
  }
}
